@@ -23,12 +23,15 @@ let load_path = ref []
 let preprocessor = ref None
 let files = ref []
 let ocamldep = ref "ocamldep"
+let sort_mli = ref false
+let no_x = ref false
+let obj = ref false
 
 let add_file file =
-  files := Filename.basename file :: !files
+  files := file :: !files
 
 let usage = 
-  "Usage: ocamldsort [-I <dir>] [-pp <command>] [-d <command>] <files>"
+  "Usage: ocamldsort [options] <files>"
 
 let is_input_a_tty () =
   try
@@ -39,12 +42,17 @@ let is_input_a_tty () =
 
 let speclist = [
   "-I", Arg.String (fun dir -> load_path := !load_path @ [dir]),
-  "<dir>  Add <dir> to the list of include directories";
+  "<dir>\tadd <dir> to the list of include directories";
   "-pp", Arg.String (fun s -> preprocessor := Some s),
-  "<command>  Pipe sources through preprocessor <command>";
+  "<command>\tpipe sources through preprocessor <command>";
   "-d", Arg.String (fun s -> ocamldep := s),
-  "<command>  Use <command> to do ocamldep job";
-]
+  "<command>\tuse <command> to do ocamldep job";
+  "-mli", Arg.Unit (fun () -> sort_mli := true),
+  "  sort files using .mli dependencies";
+  "-nox", Arg.Unit (fun () -> no_x := true),
+  "  ignore filename containing `*\'";
+  "-obj", Arg.Unit (fun () -> obj := true),
+  "  print object filename (.cmo and .cmi)" ] 
 
 let fatal_error str =
   prerr_endline str;
@@ -53,20 +61,27 @@ let fatal_error str =
 let _ =
   try
     Arg.parse speclist add_file usage; 
+    if !no_x then files := List.filter (fun x -> not (String.contains x '*')) !files;
     let input_param =
       if is_input_a_tty () then
 	Params.new_ocamldep_input_param !ocamldep !load_path !preprocessor
     else 
       Params.Stdin in
-    let param = Params.new_param input_param !files in
+    let param = Params.new_param input_param !files !sort_mli in
     let deps = Dependencies.get_dependencies param in
     let sorted = Dep_sort.sort_dependencies deps param in
-      (match sorted with
-	| [] -> ()
-	| sorted -> print_endline (String.concat " " sorted))
+      (match (if !obj then
+		List.map Files.change_ml_mli_extension sorted
+	      else 
+		sorted) with
+	 | [] -> ()
+	 | sorted -> print_endline (String.concat " " sorted))
   with 
-    | Dep_sort.Dependency_error _ ->
-	fatal_error "Error: dependency error (cyclic dependencies?)"
+    | Dep_error.Cyclic_dependency cyclic_deps ->
+	fatal_error ("Error: cyclic dependencies: " ^ 
+		     (Dep_error.string_of_cycle cyclic_deps))
+    | Dep_error.Unmet_dependency dep ->
+	fatal_error ("Error: unmet dependency: " ^ dep)
     | Files.Unknown_extension file ->
 	fatal_error ("Error: what kind of file " ^ file ^ " is?")
     | Dep_parse.Parse_error ->
